package markov;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

public class Lookup implements Serializable {

    private final LinkedHashMap<Token, Integer> tokens = new LinkedHashMap<>();
    private Decission[] finalData;
    private long[] finishedSums;
    private boolean isFinishedCollecting = false;

    private int totalCounts = 0;


    int getTotalCounts() {
        return totalCounts;
    }

    int getDistinctTokens() {
        return tokens.size();
    }

    int getAmount(Token t) {
        return tokens.getOrDefault(t, 0);
    }

    public void add(Token t) {
        if (isFinishedCollecting) {
            throw new IllegalStateException("already finished");
        }
        Integer i = tokens.getOrDefault(t, 0);
        tokens.put(t, i + 1);
        totalCounts++;
    }

    public Decission forRandom(double random) {
        if (!isFinishedCollecting) {
            throw new IllegalStateException("not jet finished");
        }
        if (random < 0 || random >= 1) {
            throw new IllegalArgumentException("expected double [0; 1)");
        }
        int id = (int) Math.floor(random * totalCounts);
        int i = Arrays.binarySearch(finishedSums, id);
        i = Math.min(i >= 0 ? i + 1 : -1 * (i + 1), finishedSums.length - 1);
        return finalData[i];
    }

    public Decission forId(int id) {
        if (!isFinishedCollecting) {
            throw new IllegalStateException("not jet finished");
        }
        if (id < 0 || id > finishedSums.length) {
            throw new IllegalArgumentException(
                    String.format("got %d but expected id [0; %d)", id, finishedSums.length));
        }
        return finalData[id];
    }

    public Decission average() {
        if (!isFinishedCollecting) {
            throw new IllegalStateException("not jet finished");
        }
        return finalData[this.finishedSums.length - 1];
    }

    public Set<Token> allPossible() {
        return this.tokens.keySet();
    }

    void finishCollection() {
        if (!isFinishedCollecting) {
            int size = tokens.size();
            finishedSums = new long[size];
            finalData = new Decission[size];
            final int[] i = new int[1];
            i[0] = 0;
            final long[] sum = new long[1];
            sum[0] = 0;
            tokens.entrySet()
                  .stream()
                  .sequential()
                  .sorted(Comparator.comparingInt(Entry::getValue))
                  .forEach(e -> {
                      sum[0] += e.getValue();
                      finishedSums[i[0]] = sum[0];
                      finalData[i[0]] = new Decission(e.getKey(), i[0], this);
                      i[0]++;
                  });
            this.tokens.clear();
            isFinishedCollecting = true;
        }
    }

    @Override
    public String toString() {
        return "Lookup [tokens= " + tokens.entrySet().stream().sorted(
                (e1, e2) -> Integer.compare(e2.getValue(), e1.getValue())).map(
                e -> String.format("%d*%s", e.getValue(), e.getKey())).collect(
                Collectors.joining(", ")) + "]";
    }

    /**
     * Not thread save!
     */
    void resetFinishding() {
        this.isFinishedCollecting = false;
        this.finishedSums = null;
        this.finalData = null;
    }

}
