package markov;

public class Renderer {
    public static class Options {
        private final boolean propability;
        private final boolean possibilities;
        private final boolean recomends;
        private final boolean specialToken;
        private final String prefix;

        public Options(boolean propability, boolean possibilities, boolean recomends, boolean specialToken, String prefix) {
            this.propability = propability;
            this.possibilities = possibilities;
            this.recomends = recomends;
            this.specialToken = specialToken;
            this.prefix = prefix;
        }

        public static final Options NONE = new Options(false, false, false, false, "");
        public static final Options FULL = new Options(true, true, true, true, "");
        public static final Options TOKEN = new Options(true, false, false, true, "");

    }

    private final Options options;

    public Renderer() {
        super();
        this.options = Options.NONE;
    }

    public Renderer(Options options) {
        super();
        this.options = options;
    }

    public String render(Sentence sentence) {
        StringBuilder sb = new StringBuilder();
        if (options.recomends)
            sb.append("KAI-uwe empfiehlt");
        for (Decission d : sentence) {
            Lookup lookup = d.getLookup();
            int possibilities = lookup.getDistinctTokens();
            if (options.possibilities) {

                if (possibilities <= 1) {
                    sb.append("-");
                } else if (possibilities <= 2) {
                    sb.append("--");
                } else if (possibilities <= 3) {
                    sb.append("---");
                } else if (possibilities <= 5) {
                    sb.append("_");
                } else if (possibilities <= 10) {
                    sb.append("__");
                } else if (possibilities <= 20) {
                    sb.append("___");
                } else if (possibilities <= 50) {
                    sb.append("*");
                } else if (possibilities <= 100) {
                    sb.append("**");
                } else if (possibilities <= 200) {
                    sb.append("***");
                } else {
                    sb.append(" ");
                }
            }
            if ((Token.START .equals(d.getToken())||  Token.END.equals(d.getToken()) ) && !options.specialToken) {
                continue;
            }
            sb.append(d.getToken().render(options.prefix));
        }
        if (options.propability) {
            sb.append(" - ").append(sentence.propability());
        }
        return sb.toString();
    }


}
