package markov;

import markov.huffman.HuffmanCode;
import markov.stuff.BitConverter;
import markov.stuff.SimpleCountMap;
import markov.stuff.Utils;

import java.util.Base64;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ShortenerByteHuffmanImpl extends ShortenerSimpleImpl {


    private final HuffmanCode<Byte, List<Boolean>, Boolean, SimpleCountMap.MutableInt> byteCode;


    public ShortenerByteHuffmanImpl(
            Data data,
            HuffmanCode<Byte, List<Boolean>, Boolean, SimpleCountMap.MutableInt> byteCode) {
        super(data);
        this.byteCode = byteCode;
    }

    String fromInts(Stream<Integer> ints) {
        List<Boolean> bl = ints.flatMap(i -> {
            if (i < 128) {
                return Stream.of((byte) (i | 0b10000000));
            } else {
                Stream.Builder<Byte> builder = Stream.<Byte>builder();
                for (byte b : Utils.toByteArray(i)) {
                    builder.add(b);
                }
                return builder.build();
            }
        }).flatMap(b -> byteCode.encode(b).stream()).collect(Collectors.toList());
        BitConverter bc = new BitConverter();
        return Base64.getEncoder().encodeToString(bc.toBytes(bl));
    }

    @Override
    Stream<Integer> toInts(String hash) {
        Stream.Builder<Integer> builder = Stream.builder();
        List<Byte> decoded = byteCode.decode(
                new BitConverter().toBits(Base64.getDecoder().decode(hash)));
        Iterator<Byte> iter = decoded.iterator();
        while (iter.hasNext()) {
            byte cur = iter.next();
            if (cur < 128) {
                builder.add((int) cur);
            } else {
                byte[] tmp = new byte[4];
                tmp[0] = cur;
                tmp[1] = iter.next();
                tmp[2] = iter.next();
                tmp[3] = iter.next();
                builder.add(Utils.fromByteArray(tmp));
            }
        }
        return builder.build();
    }
}