package markov;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Tokenizer {


    public Stream<Stream<Token>> tokenizeCombined(Stream<Stream<String>> input) {
        return input.map(stringStream -> putMarkers(combineToTokens(stringStream.flatMap(s -> s.codePoints().boxed().map(


                this::glyphFromCodePoint)))));
    }

    public Stream<Stream<Token>> tokenize(Stream<String> input) {
        return input.map(s -> putMarkers(combineToTokens(s.codePoints().boxed().map(
                this::glyphFromCodePoint))));
    }

    private Glyph glyphFromCodePoint(int codePoint) {
        Glyph.Type type = Glyph.Type.other;
        if (Character.isWhitespace(codePoint)) {
            type = Glyph.Type.whitespace;
        } else if (Character.isAlphabetic(codePoint)) {
            type = Glyph.Type.word;
        } else if (Arrays.asList(Character.START_PUNCTUATION, Character.END_PUNCTUATION, Character.INITIAL_QUOTE_PUNCTUATION, Character.FINAL_QUOTE_PUNCTUATION,
                Character.OTHER_PUNCTUATION).contains(Character.getType(codePoint))) {
            type = Glyph.Type.punctuation;
        }
        String value = new String(Character.toChars(codePoint));
        return new Glyph(type, value);
    }

    private Stream<Token> putMarkers(Stream<Token> input) {
        return Stream.concat(Stream.of(Token.START), Stream.concat(input, Stream.of(Token.END)));
    }

    public Stream<Token> combineToTokens(Stream<Glyph> glyphs) {
        final Container[] previous = new Container[]{null};
        return Stream.concat(glyphs, Stream.of(new Glyph(Glyph.Type.empty, "")))
                .map(g -> {
                    Container o = new Container(g);
                    o.p = previous[0];
                    previous[0] = o;
                    return o;
                }).flatMap(go -> {
                    if (go.p != null && go.p.self != null && !go.self.getType().equals(go.p.self.getType())) {
                        List<Glyph> tokenGlyphs = new ArrayList<>();
                        Container c = go;
                        do {
                            c = c.p;
                            tokenGlyphs.add(0, c.self);
                        }
                        while (c != null && c.p != null && c.p.self != null &&
                                c.self.getType().equals(c.p.self.getType()));
                        go.p = null; // memory optimization
                        return Stream.<List<Glyph>>builder().add(tokenGlyphs).build();
                    } else {
                        return Stream.empty();
                    }
                }).map(this::getTokenFromGlyphs);
    }

    private Token getTokenFromGlyphs(List<Glyph> l) {
        String content = l.stream().map(Glyph::getContent).collect(Collectors.joining());
        Glyph.Type type = l.get(0).getType();
        return new Token(content, type);
    }

    private static class Container {
        Container(Glyph self) {
            this.self = self;
        }

        final Glyph self;
        Container p;
    }
}
