package markov.huffman;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Function;

public class HuffmanCode<ContentType, SequenceType, GlyphType, FrequencyType extends FrequenceType<FrequencyType>> implements Serializable {


    private final Map<ContentType, SequenceType> codes;
    private final HuffmanTree<ContentType, SequenceType, FrequencyType> tree;
    private final Function<SequenceType, Iterator<GlyphType>> splitter;

    private final BiFunction<HuffmanNode<ContentType, SequenceType, FrequencyType>,
            GlyphType, HuffmanTree<ContentType, SequenceType, FrequencyType>> leftRightDecider;

    public HuffmanCode(Map<ContentType, SequenceType> codes,
                       HuffmanTree<ContentType, SequenceType, FrequencyType> tree,
                       Function<SequenceType, Iterator<GlyphType>> splitter,
                       BiFunction<HuffmanNode<ContentType, SequenceType, FrequencyType>,
                               GlyphType,
                               HuffmanTree<ContentType, SequenceType, FrequencyType>> leftRightDecider) {
        this.codes = codes;
        this.tree = tree;
        this.splitter = splitter;
        this.leftRightDecider = leftRightDecider;
    }

    public SequenceType encode(ContentType t) {
        return codes.get(t);
    }

    public List<ContentType> decode(SequenceType v) {
        List<ContentType> ret = new ArrayList<>();
        Iterator<GlyphType> iterator = splitter.apply(v);
        HuffmanTree<ContentType, SequenceType, FrequencyType> node = tree;
        while (iterator.hasNext()) {
            GlyphType glyph = iterator.next();
            node = leftRightDecider.apply((HuffmanNode<ContentType, SequenceType, FrequencyType>) node, glyph);
            if (node instanceof HuffmanLeaf) {
                ret.add(((HuffmanLeaf<ContentType, SequenceType, FrequencyType>) node).getValue());
                node = tree;
            }
        }
        return ret;
    }
}