package markov.stuff;

import markov.huffman.FrequenceType;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Stream;

public class SimpleCountMap<T> extends HashMap<T, SimpleCountMap.MutableInt> implements CountMap<T> {
    public static class MutableInt implements FrequenceType<MutableInt>, Serializable {
        private int value = 1;

        public MutableInt(int value) {
            this.value = value;
        }

        public void increment() {
            ++value;
        }

        public int get() {
            return value;
        }

        @Override
        public MutableInt add(MutableInt b) {
            return new MutableInt(get() + b.get());
        }

        @Override
        public boolean isGreaterZero() {
            return get() > 0;
        }

        @Override
        public int compareTo(MutableInt o) {
            return Integer.compare(get(), o.get());
        }
    }

    public SimpleCountMap(int initialCapacity, float loadFactor) {
        super(initialCapacity, loadFactor);
    }

    public SimpleCountMap(int initialCapacity) {
        super(initialCapacity);
    }

    public SimpleCountMap() {
    }

    public SimpleCountMap(Map<? extends T, ? extends MutableInt> m) {
        super(m);
    }

    @Override
    public void count(T t) {
        MutableInt i = get(t);
        if (i == null) {
            put(t, new MutableInt(1));
        } else {
            i.increment();
        }
    }

    @Override
    public Stream<Map.Entry<T, MutableInt>> result() {
        return entrySet().stream().sorted((e1, e2) -> Integer.compare(e2.getValue().get(), e1.getValue().get()));
    }

    @Override
    public HashMap<T, MutableInt> asMap() {
        return this;
    }
}