package markov;

import org.junit.jupiter.api.Test;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class PrefixTests {

    Prefix prefix;

    @Test
    public void create() {
        prefix = new Prefix(Arrays.asList(new Token("a")));
        assertNotNull(prefix);
        assertEquals(new Prefix(Arrays.asList(new Token("a"))), prefix);
    }

    @Test
    public void testSlide() {
        prefix = new Prefix(Arrays.asList(new Token("a")));
        Prefix p2 = prefix.slide(new Token("b"));
        assertEquals(new Prefix(Arrays.asList(new Token("b"))), p2);

        prefix = new Prefix(Arrays.asList(new Token("a"), new Token("b")));
        p2 = prefix.slide(new Token("c"));
        assertEquals(new Prefix(Arrays.asList(new Token("b"), new Token("c"))), p2);
    }

    @Test
    public void testFancySlide() {
        prefix = new Prefix(Arrays.asList(new Token("a")));
        Prefix p2 = prefix.slide(new Token("b"), 2);
        assertEquals(new Prefix(Arrays.asList(new Token("a"), new Token("b"))), p2);
    }

    @Test
    void testExplicitGrow() {
        prefix = new Prefix(Arrays.asList(new Token("a"), new Token("b")));
        Prefix p2 = prefix.slide(new Token("c"), 3);
        assertEquals(new Prefix(Arrays.asList(new Token("a"), new Token("b"), new Token("c"))
        ), p2);
    }

    @Test
    void testToLargeGrow() {
        prefix = new Prefix(Arrays.asList(new Token("a"), new Token("b")));
        Prefix p2 = prefix.slide(new Token("c"), 4);
        assertEquals(new Prefix(Arrays.asList(new Token("a"), new Token("b"), new Token("c"))
        ), p2);
    }

    @Test
    public void testShrink() {
        prefix = new Prefix(Arrays.asList(new Token("a"), new Token("b"), new Token("c")));
        Prefix p2 = prefix.slide(new Token("d"), 2);
        assertEquals(new Prefix(Arrays.asList(new Token("c"), new Token("d"))), p2);
    }


}